(***********************************************************************)
(*                                                                     *)
(*                               Grimage                               *)
(*                                                                     *)
(*                           Graphics image                            *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2014-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ppm.mli,v 1.9 2014/03/08 12:23:47 fclement Exp $ *)

(** Loading and saving images in portable format: PPM, PGM, and PBM.

PPM: portable pixmap (pixels (picture element) map).
PGM: portable greymap (grey scale map).
PBM: portable bitmap (binary digit map (black & white images)).

*)

(** In short:

- [save_image fname img] saves an image from the graphics
  library as a PPM image in file [fname].
- [load_image fname] loads a PPM image stored in file [fname] as
  a graphics image. *)

open Generic_image;;

type ppm = pixmap;;

type ppm_magic_number = | P1 | P2 | P3 | P4 | P5 | P6;;
 (** Magic numbers for PPM images.
    P1 and P4 indicate bitmaps;
    P2 and P5 indicate greymaps;
    P3 and P6 indicate pixmaps.
    P1, P2, P3 indicate ascii encoding;
    P4, P5, P6 indicate raw encoding.

    The library systematically saves images in raw form (which is more compact).
 *)

val ppm_magic_number : file_name -> ppm_magic_number;;

val generic_image_of_ppm : ppm -> Generic_image.image;;
val ppm_of_generic_image : Generic_image.image -> ppm;;
  (** Conversion from [ppm] images to generic images of
      the [Grimage] library. *)

val image_of_ppm : ppm -> Graphics.image;;
  (** Another name for [Pixmap.image_of_pixmap]. *)
val ppm_of_image : Graphics.image -> ppm;;
  (** Another name for [Pixmap.pixmap_of_image]. *)

val load_ppm : file_name -> ppm;;
 (** [load_ppm fname] reads the image contained in the file [fname],
    and returns a [pixmap], i.e. an array of integers (representing
    the RGB colors of pixels). Assumes that the image is stored in
    one of the ppm formats, that is either PPM (Portable Pixmap), PGM
    (Portable Greymap), or PBM (Portable Bitmap) formats. The PPM format
    is 24 bits per pixel, and both ASCII and raw encoding are
    supported. The PGM format is 8 bits per pixel, and both ASCII and raw
    encoding are supported. The PBM is 1 bit per pixel, and both ASCII
    and raw encoding are supported. If a PPM image is followed by a
    PBM image in the same file, the PBM image is supposed to be the
    transparency mask of the first image : pixels set to [1] in the
    mask receive the [transp] color into the resulting image (see
    [save_ppm] below). *)

val save_ppm : file_name -> ppm -> unit;;
 (** [save_ppm fname ppm] stores the image [ppm], in file [fname].
    Argument [ppm] is a [pixmap], that is an image supposed to be
    represented as an array of colors in the spirit of images of the
    [Graphics] library. The image is saved as a portable ppm, in raw
    encoding mode. If the image contains transparent pixels
    (i.e. pixels with color set to [Graphics.transp]), an extra
    transparency mask is saved in the file [fname], as a PBM
    image. (See [load_ppm] above.)  *)

val save_bitmap : file_name -> ppm -> unit;;
val load_bitmap : file_name -> ppm;;
 (** Same as above for portable bitmaps. *)

val load_pixmap : file_name -> pixmap;;
val save_pixmap : file_name -> pixmap -> unit;;
 (** Equivalent to [load_ppm] and [save_ppm]. *)

val load_generic_image : file_name -> Generic_image.image;;
val save_generic_image : file_name -> Generic_image.image -> unit;;
(** Same as above for generic images. *)

val load_image : file_name -> Graphics.image;;
 (** [load_image fname] loads the PPM image in file [fname] as an
    image of the graphics library. *)

val save_image : file_name -> Graphics.image -> unit;;
 (** [save_image fname img] saves the image [img] of the graphics
    library as a PPM image in file [fname]. *)
